/* stencil-teapot-i.c */

#include <math.h>
#include "LoadTGA.h"
#include "VectorUtils3.h"
#include "GL_utilities.h"
#include "LittleOBJLoader.h"
#include "MicroGlut.h"
// uses framework Cocoa
// uses framework OpenGL

	GLubyte minitex[4][4][3] =
	{
		{ {255,  0,255}, {  0,  0,255}, {  0,  0,255}, {  0,255,255}},
		{ {  0,  0,255}, {255,  0,255}, {  0,255,255}, {  0,  0,255}},
		{ {  0,  0,255}, {  0,255,255}, {255,  0,255}, {  0,  0,255}},
		{ {  0,255,255}, {  0,  0,255}, {  0,  0,255}, {255,  0,255}},
	};

long count = 0;
int mode = 0;
int t = 0;
char solid = 0;

GLuint program;
mat4 projectionMatrix, modelViewMatrix;
Model *teapot, *sphere;

void display()
{
	mat4 mv2, tx2, trans, rot, scale;
	
	// Clear frame buffer, Z buffer and stencil buffer
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT); // <- ***
	
	count++;

	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	
	Point3D axis = {1, 1, 1};
	rot = ArbRotate(axis, 0.2 * count*0.1);
	mv2 = Mult(modelViewMatrix, rot);
	glUniformMatrix4fv(glGetUniformLocation(program, "modelViewMatrix"), 1, GL_TRUE, mv2.m);
	
	glUniform1i(glGetUniformLocation(program, "t"), t);

	if (mode == 0)
	{
		// Enable the Stencil Buffer
		glEnable(GL_STENCIL_TEST);

		// Disable Color Buffer and Depth Buffer
		glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
		glDepthMask(GL_FALSE);

		// Set 1 into the stencil buffer
		glStencilFunc(GL_ALWAYS, 1, 0xFFFFFFFF);
		glStencilOp(GL_REPLACE, GL_REPLACE, GL_REPLACE);
		// Draw the wired sphere in the stencil buffer only
		if (solid)
			DrawModel(sphere, program, "in_Position", NULL, "inTexCoord");
		else
			DrawWireframeModel(sphere, program, "in_Position", NULL, "inTexCoord");

		// Turn on Color Buffer and Depth Buffer
		glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
		glDepthMask(GL_TRUE);

		// Only write to the Stencil Buffer where 1 is set
		glStencilFunc(GL_EQUAL, 1, 0xFFFFFFFF);
		// Keep the content of the Stencil Buffer
		glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
		
		// Draw the shape
		DrawModel(teapot, program, "in_Position", NULL, "inTexCoord");

		glDisable(GL_STENCIL_TEST); // Turn off (in case we add other stuff that does not use it)
	}


	if (mode == 1)
	{
	// Draw the shape
		if (solid)
			DrawModel(teapot, program, "in_Position", NULL, "inTexCoord");
		else
			DrawWireframeModel(teapot, program, "in_Position", NULL, "inTexCoord");
	}
	
	if (mode == 2)
	{
		if (solid)
			DrawModel(sphere, program, "in_Position", NULL, "inTexCoord");
		else
			DrawWireframeModel(sphere, program, "in_Position", NULL, "inTexCoord");
	}
		
	glutSwapBuffers();
}

Point3D p_camera = {0, 0, 12.5};
Point3D l_camera = {0, 0, 0};

void reshape(GLsizei w, GLsizei h)
{
	glViewport(0, 0, w, h);
	projectionMatrix = perspective(90, w/h, 1, 40);
	
	modelViewMatrix = lookAt(p_camera.x, p_camera.y, p_camera.z,
										l_camera.x, l_camera.y, l_camera.z,
										0,1,0);
}

void timer(int i)
{
	glutTimerFunc(20, timer, i);
	glutPostRedisplay();
}

void key(unsigned char key, int x, int y)
{
	switch (key)
	{
		case 'z':
				mode = 0;
			break;
		case 'x':
				mode = 1;
			break;
		case 'c':
				mode = 2;
			break;
		case 't':
				t = 1-t;
			break;
		case 's':
				solid = !solid;
			break;
	}
}

GLuint texID;

int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH | GLUT_STENCIL); // <- ***
	glutInitContextVersion(3, 2);
	glutCreateWindow("simple stencil demo zxc t s");
	glutDisplayFunc(display);
	glutTimerFunc(20, timer, 43);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(key);
	
	glEnable(GL_DEPTH_TEST);
	glClearColor(0,0,0,1.0f);

	glGenTextures(1, &texID);			// Generate OpenGL texture IDs
	glBindTexture(GL_TEXTURE_2D, texID);		// Bind Our Texture
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 4, 4, 0, GL_RGB, GL_UNSIGNED_BYTE, minitex);

	
	program = loadShaders("tex.vert", "tex.frag");
	teapot = LoadModelPlus("teapot.obj");
	sphere = LoadModelPlus("sphere3.obj");

	glUniform1i(glGetUniformLocation(program,"texUnit"),0);
	
	modelViewMatrix = lookAt(p_camera.x, p_camera.y, p_camera.z,
										l_camera.x, l_camera.y, l_camera.z,
										0,1,0);
	projectionMatrix = perspective(45, 1, 10, 4000);
	
	glutMainLoop();
	exit(0);
}
